--[[

	BagSaver addon. 
	Primary functions are auto-selling unwanted items
	to a vendor, finding the least valuable item in 
	your bags to discard and more.
	Author: OneWingedAngel

--]]


--[[ Globals --]]
BagSaverConfig = {}

--[[ Main addon frame creation and main local vars ]]--
local BagSaver = CreateFrame("Frame", "BagSaverFrame", UIParent)

local ScanningTooltip = CreateFrame( "GameTooltip", "ScanningTooltipFrame" )
ScanningTooltip:SetOwner( WorldFrame, "ANCHOR_NONE" )
ScanningTooltip:AddFontStrings(
	ScanningTooltip:CreateFontString( "$parentTextLeft1", nil, "GameTooltipText" ),
	ScanningTooltip:CreateFontString( "$parentTextRight1", nil, "GameTooltipText" ) )

local UnusableEquipment = {}
local DefaultConfig = {}
local RegisteredEvents = {}
BagSaverFrame:SetScript("OnEvent", function (self, event, ...) if (RegisteredEvents[event]) then return RegisteredEvents[event](self, event, ...) end end)

--[[ Event Handlers --]]

function RegisteredEvents:ADDON_LOADED(event, addon, ...)
	if (addon == "BagSaver") then
		SLASH_BAGSAVER1, SLASH_BAGSAVER2 = '/bagsaver', '/bs'
		SlashCmdList["BAGSAVER"] = function (msg, editbox)
			BagSaver.SlashCmdHandler(msg, editbox)	
		end

		--Use metatable to implement defaults, see http://www.lua.org/pil/13.4.1.html
		setmetatable(BagSaverConfig, {__index = DefaultConfig})

		BagSaver:CreateConfigMenu()
		print("BagSaver " .. GetAddOnMetadata("BagSaver","Version") .. " Loaded")
	end
end

function RegisteredEvents:MERCHANT_SHOW()
	if BagSaver.GetConfigValue("autoSell") then
		local itemsToSell, numItemsToSell = BagSaver:GetJunkInBags()

		if numItemsToSell ~= 0 then
			if BagSaver.GetConfigValue("autoSellPrompt") then
				BagSaver.PromptToSellItemsToVendor(itemsToSell)
			else
				BagSaver.SellItemsToVendor(itemsToSell)
			end
		end
	end
end

function RegisteredEvents:LOOT_OPENED(autoloot)
	local numItems = GetNumLootItems()
end

for k, v in pairs(RegisteredEvents) do
	BagSaver:RegisterEvent(k)
end

--[[ Addon functions --]]
function BagSaver.SellItemsToVendor(items)
	for itemQuality=0,7 do
		for itemLink,v in pairs(items[itemQuality]) do
			local newTexture, newCount, newLocked, newQuality, newReadable, newLootable, newItemLink = GetContainerItemInfo(items[itemQuality][itemLink]["bag"], items[itemQuality][itemLink]["slot"]) --Check to make sure the items haven't moved since we looked
			if (itemLink == newItemLink) then
				if items[itemQuality][itemLink]["count"] == 1 then
					print("BagSaver auto-selling: " .. newItemLink)
				else
					print("BagSaver auto-selling: " .. newCount .. "x " .. newItemLink)
				end
				UseContainerItem(items[itemQuality][itemLink]["bag"],items[itemQuality][itemLink]["slot"])
			else
				print(itemLink .. " has moved! Not attempting to sell " .. newItemLink)
			end
		end
	end
end

function BagSaver.PromptToSellItemsToVendor(items)
	local itemLinks = {}
	for itemQuality=0,7 do
		for itemLink,v in pairs(items[itemQuality]) do
			if items[itemQuality][itemLink]["count"] == 1 then
				table.insert(itemLinks,itemLink)
			else
				table.insert(itemLinks,items[itemQuality][itemLink]["count"] .. "x " .. itemLink)
			end
		end
	end

	local dialogText = "Sell the following items?\n"
	StaticPopupDialogs["BAGSAVER_AUTOSELL_PROMPT"] = {
		text = dialogText .. table.concat(itemLinks, "\n"),
		button1 = "Yes",
		button2 = "No",
		OnAccept = function() BagSaver.SellItemsToVendor(items) end,
		timeout = 0,
		whileDead = 1,
		hideOnEscape = 1,
		showAlert = 1
	}
	StaticPopup_Show("BAGSAVER_AUTOSELL_PROMPT")
end

function BagSaver.GetJunkInBags()
	local items = {}
	for itemQuality=0,7 do
		items[itemQuality] = {}
	end
	local itemsFound = 0
	local playerClassLocal, playerClass = UnitClass("player")
	for bag = 0,NUM_BAG_SLOTS do
		for slot = 1,GetContainerNumSlots(bag) do
			local texture, count, locked, _, readable, lootable, link = GetContainerItemInfo(bag,slot)
			local id = GetContainerItemID(bag,slot)
			if id then --We're not looking at an empty slot
				local name, _, quality, iLevel, reqLevel, class, subClass, maxStack, equipSlot, _, vendorPrice = GetItemInfo(id)
				if (vendorPrice > 0) then --We don't care about items we can't sell
					--print("Working on " .. link,quality,class,subClass,vendorPrice)
					if (BagSaver.GetConfigValue("autoSellGrayItems")) then
						if quality == 0 then
							--print("Found " .. link .. " which is " .. quality .. " quality in bag " .. bag .. " slot " .. slot)
							BagSaver.AddItemToList(items,quality,link,bag,slot,count,vendorPrice)
							itemsFound = itemsFound + 1
						end
					end
					if (BagSaver.GetConfigValue("autoSellUnusableBoundItems")) then
						if (BagSaver.ItemIsSoulbound(bag,slot) and UnusableEquipment[playerClass][class] ~= nil and UnusableEquipment[playerClass][class][subClass] ~= nil) then --item is soulbound and could never be equipped by the player's class
							BagSaver.AddItemToList(items,quality,link,bag,slot,count,vendorPrice)
							itemsFound = itemsFound + 1
						end
					end
				end
			end
		end
	end

	return items, itemsFound
end

function BagSaver.ItemIsSoulbound(bag,slot)
	ScanningTooltip:ClearLines()
	ScanningTooltip:SetBagItem(bag,slot)

	--print(getglobal("ScanningTooltipFrameTextLeft2"):GetText())

	if (getglobal("ScanningTooltipFrameTextLeft2"):GetText() == ITEM_SOULBOUND) then
		--print("Item is Soulbound!")
		return true
	end
	return false
end

function BagSaver.AddItemToList(list,quality,itemLink,bag,slot,count,vendorPrice)
	--print("Got " .. quality,itemLink,bag,slot,count,vendorPrice)
	list[quality][itemLink] = {}
	list[quality][itemLink]["bag"] = bag
	list[quality][itemLink]["slot"] = slot
	list[quality][itemLink]["count"] = count
	list[quality][itemLink]["vendorPrice"] = vendorPrice
end

function BagSaver.SlashCmdHandler(msg, editbox)
	--print("command is " .. msg .. "\n")
	if (string.lower(msg) == "debug") then
		DEBUG = not DEBUG
	elseif (string.lower(msg) == "config") then
		InterfaceOptionsFrame_OpenToCategory("BagSaver")
	elseif (string.lower(msg) == "dumpconfig") then
		print("With defaults")
		for k,v in pairs(DefaultConfig) do
			print(k,BagSaver.GetConfigValue(k))
		end
		print("Direct table")
		for k,v in pairs(BagSaverConfig) do
			print(k,v)
		end
	elseif (string.lower(msg) == "foo") then
		BagSaver.GetJunkInBags()
	elseif (string.lower(msg) == "dumpequip") then
		BagSaver.DumpEquipmentTable()
	elseif (string.lower(msg) == "reset") then
		BagSaver.SetConfigToDefaults()
	else
		BagSaver.ShowHelp()
	end
end

function BagSaver.ShowHelp()
	print("Slash commands (/bagsaver or /bs):")
	print(" /bs config:	Open addon config menu (also found in Addon tab in Blizzard's Interface menu)")
	print(" /bs reset:	Resets your config to defaults")
end

function BagSaver.DumpEquipmentTable()
	for className, localizedClassName in pairs(LOCALIZED_CLASS_NAMES_MALE) do
		print(className)
		for itemType,v in pairs(UnusableEquipment[className]) do
			print(" " .. itemType)
			for itemSubType,v in pairs(UnusableEquipment[className][itemType]) do
				print("  " .. itemSubType,v)
			end
		end
	end
end

--[[ Configuration methods --]]
function BagSaver.GetConfigValue(key)
	return BagSaverConfig[key]
end

function BagSaver.SetConfigValue(key, value)
	if (DefaultConfig[key] == value) then
		BagSaverConfig[key] = nil
	else 
		BagSaverConfig[key] = value
	end
end

function BagSaver.SetConfigToDefaults()
	print("Resetting config to defaults")
	BagSaverConfig = {}
	setmetatable(BagSaverConfig, {__index = DefaultConfig})
end

--[[ "Global" Vars --]]

DefaultConfig = {
	autoSell = true,
	autoSellPrompt = true,
	autoSellGrayItems = true,
	autoSellUnusableBoundItems = false,
}

--Define local vars with localized name of each weapon and armor type

--[[
for classIndex,className in ipairs({GetAuctionItemClasses()}) do
	print(classIndex,className)
	for subClassIndex,subClassName in ipairs({GetAuctionItemSubClasses(classIndex)}) do
		print(" " .. subClassIndex,subClassName)
	end
end
--]]

local WEAPON,ARMOR,CONTAINER,CONSUMABLE,GLYPH,TRADEGOODS,PROJECTILE,QUIVER,RECIPE,GEM,MISCELLANEOUS,QUEST = GetAuctionItemClasses()
local ONEHANDAXES,TWOHANDAXES,BOWS,GUNS,ONEHANDMACES,TWOHANDMACES,POLEARMS,ONEHANDSWORDS,TWOHANDSWORDS,STAVES,FISTWEAPONS,WEAPONMISC,DAGGERS,THROWN,CROSSBOWS,WANDS,FISHINGPOLES = GetAuctionItemSubClasses(1)
local ARMORMISC,CLOTH,LEATHER,MAIL,PLATE,SHIELDS,LIBRAMS,IDOLS,TOTEMS,SIGILS = GetAuctionItemSubClasses(2)
--print(WEAPON,ARMOR,CONTAINER,CONSUMABLE,GLYPH,TRADEGOODS,PROJECTILE,QUIVER,RECIPE,GEM,MISCELLANEOUS,QUEST)
--print(ONEHANDAXES,TWOHANDAXES,BOWS,GUNS,ONEHANDMACES,TWOHANDMACES,POLEARMS,ONEHANDSWORDS,TWOHANDSWORDS,STAVES,FISTWEAPONS,WEAPONMISC,DAGGERS,THROWN,CROSSBOWS,WANDS,FISHINGPOLES)
--print(ARMORMISC,CLOTH,LEATHER,MAIL,PLATE,SHIELDS,LIBRAMS,IDOLS,TOTEMS,SIGILS)

--Initialize our table structure for each class
UnusableEquipment["CLASS"] = {}
UnusableEquipment["CLASS"][WEAPON] = {}
UnusableEquipment["CLASS"][ARMOR] = {}
for className, localizedClassName in pairs(LOCALIZED_CLASS_NAMES_MALE) do
	UnusableEquipment[className] = {}
	UnusableEquipment[className][WEAPON] = {}
	UnusableEquipment[className][ARMOR] = {}
end

--Placeholder for all equipment and armor types
--[[
UnusableEquipment["CLASS"][WEAPON][BOWS] = 1
UnusableEquipment["CLASS"][WEAPON][CROSSBOWS] = 1
UnusableEquipment["CLASS"][WEAPON][DAGGERS] = 1
UnusableEquipment["CLASS"][WEAPON][FISTWEAPONS] = 1
UnusableEquipment["CLASS"][WEAPON][GUNS] = 1
UnusableEquipment["CLASS"][WEAPON][ONEHANDAXES] = 1
UnusableEquipment["CLASS"][WEAPON][ONEHANDMACES] = 1
UnusableEquipment["CLASS"][WEAPON][ONEHANDSWORDS] = 1
UnusableEquipment["CLASS"][WEAPON][POLEARMS] = 1
UnusableEquipment["CLASS"][WEAPON][STAVES] = 1
UnusableEquipment["CLASS"][WEAPON][THROWN] = 1
UnusableEquipment["CLASS"][WEAPON][TWOHANDAXES] = 1
UnusableEquipment["CLASS"][WEAPON][TWOHANDMACES] = 1
UnusableEquipment["CLASS"][WEAPON][TWOHANDSWORDS] = 1
UnusableEquipment["CLASS"][WEAPON][WANDS] = 1
UnusableEquipment["CLASS"][ARMOR][CLOTH] = 1
UnusableEquipment["CLASS"][ARMOR][LEATHER] = 1
UnusableEquipment["CLASS"][ARMOR][MAIL] = 1
UnusableEquipment["CLASS"][ARMOR][PLATE] = 1
UnusableEquipment["CLASS"][ARMOR][SHIELDS] = 1
UnusableEquipment["CLASS"][ARMOR][LIBRAMS] = 1
UnusableEquipment["CLASS"][ARMOR][IDOLS] = 1
UnusableEquipment["CLASS"][ARMOR][TOTEMS] = 1
UnusableEquipment["CLASS"][ARMOR][SIGILS] = 1
--]]

UnusableEquipment["DEATHKNIGHT"][WEAPON][BOWS] = 1
UnusableEquipment["DEATHKNIGHT"][WEAPON][CROSSBOWS] = 1
UnusableEquipment["DEATHKNIGHT"][WEAPON][DAGGERS] = 1
UnusableEquipment["DEATHKNIGHT"][WEAPON][FISTWEAPONS] = 1
UnusableEquipment["DEATHKNIGHT"][WEAPON][GUNS] = 1
UnusableEquipment["DEATHKNIGHT"][WEAPON][STAVES] = 1
UnusableEquipment["DEATHKNIGHT"][WEAPON][THROWN] = 1
UnusableEquipment["DEATHKNIGHT"][WEAPON][WANDS] = 1
UnusableEquipment["DEATHKNIGHT"][ARMOR][SHIELDS] = 1
UnusableEquipment["DEATHKNIGHT"][ARMOR][LIBRAMS] = 1
UnusableEquipment["DEATHKNIGHT"][ARMOR][IDOLS] = 1
UnusableEquipment["DEATHKNIGHT"][ARMOR][TOTEMS] = 1

UnusableEquipment["DRUID"][WEAPON][BOWS] = 1
UnusableEquipment["DRUID"][WEAPON][CROSSBOWS] = 1
UnusableEquipment["DRUID"][WEAPON][GUNS] = 1
UnusableEquipment["DRUID"][WEAPON][ONEHANDAXES] = 1
UnusableEquipment["DRUID"][WEAPON][ONEHANDSWORDS] = 1
UnusableEquipment["DRUID"][WEAPON][THROWN] = 1
UnusableEquipment["DRUID"][WEAPON][TWOHANDAXES] = 1
UnusableEquipment["DRUID"][WEAPON][TWOHANDSWORDS] = 1
UnusableEquipment["DRUID"][WEAPON][WANDS] = 1
UnusableEquipment["DRUID"][ARMOR][MAIL] = 1
UnusableEquipment["DRUID"][ARMOR][PLATE] = 1
UnusableEquipment["DRUID"][ARMOR][SHIELDS] = 1
UnusableEquipment["DRUID"][ARMOR][LIBRAMS] = 1
UnusableEquipment["DRUID"][ARMOR][TOTEMS] = 1
UnusableEquipment["DRUID"][ARMOR][SIGILS] = 1

UnusableEquipment["HUNTER"][WEAPON][ONEHANDMACES] = 1
UnusableEquipment["HUNTER"][WEAPON][TWOHANDMACES] = 1
UnusableEquipment["HUNTER"][WEAPON][WANDS] = 1
UnusableEquipment["HUNTER"][ARMOR][PLATE] = 1
UnusableEquipment["HUNTER"][ARMOR][SHIELDS] = 1
UnusableEquipment["HUNTER"][ARMOR][LIBRAMS] = 1
UnusableEquipment["HUNTER"][ARMOR][IDOLS] = 1
UnusableEquipment["HUNTER"][ARMOR][TOTEMS] = 1
UnusableEquipment["HUNTER"][ARMOR][SIGILS] = 1

UnusableEquipment["MAGE"][WEAPON][BOWS] = 1
UnusableEquipment["MAGE"][WEAPON][CROSSBOWS] = 1
UnusableEquipment["MAGE"][WEAPON][FISTWEAPONS] = 1
UnusableEquipment["MAGE"][WEAPON][GUNS] = 1
UnusableEquipment["MAGE"][WEAPON][ONEHANDAXES] = 1
UnusableEquipment["MAGE"][WEAPON][ONEHANDMACES] = 1
UnusableEquipment["MAGE"][WEAPON][POLEARMS] = 1
UnusableEquipment["MAGE"][WEAPON][THROWN] = 1
UnusableEquipment["MAGE"][WEAPON][TWOHANDAXES] = 1
UnusableEquipment["MAGE"][WEAPON][TWOHANDMACES] = 1
UnusableEquipment["MAGE"][WEAPON][TWOHANDSWORDS] = 1
UnusableEquipment["MAGE"][ARMOR][LEATHER] = 1
UnusableEquipment["MAGE"][ARMOR][MAIL] = 1
UnusableEquipment["MAGE"][ARMOR][PLATE] = 1
UnusableEquipment["MAGE"][ARMOR][SHIELDS] = 1
UnusableEquipment["MAGE"][ARMOR][LIBRAMS] = 1
UnusableEquipment["MAGE"][ARMOR][IDOLS] = 1
UnusableEquipment["MAGE"][ARMOR][TOTEMS] = 1
UnusableEquipment["MAGE"][ARMOR][SIGILS] = 1

UnusableEquipment["PALADIN"][WEAPON][BOWS] = 1
UnusableEquipment["PALADIN"][WEAPON][CROSSBOWS] = 1
UnusableEquipment["PALADIN"][WEAPON][DAGGERS] = 1
UnusableEquipment["PALADIN"][WEAPON][FISTWEAPONS] = 1
UnusableEquipment["PALADIN"][WEAPON][GUNS] = 1
UnusableEquipment["PALADIN"][WEAPON][STAVES] = 1
UnusableEquipment["PALADIN"][WEAPON][THROWN] = 1
UnusableEquipment["PALADIN"][WEAPON][WANDS] = 1
UnusableEquipment["PALADIN"][ARMOR][IDOLS] = 1
UnusableEquipment["PALADIN"][ARMOR][TOTEMS] = 1
UnusableEquipment["PALADIN"][ARMOR][SIGILS] = 1

UnusableEquipment["PRIEST"][WEAPON][BOWS] = 1
UnusableEquipment["PRIEST"][WEAPON][CROSSBOWS] = 1
UnusableEquipment["PRIEST"][WEAPON][FISTWEAPONS] = 1
UnusableEquipment["PRIEST"][WEAPON][GUNS] = 1
UnusableEquipment["PRIEST"][WEAPON][ONEHANDAXES] = 1
UnusableEquipment["PRIEST"][WEAPON][ONEHANDSWORDS] = 1
UnusableEquipment["PRIEST"][WEAPON][POLEARMS] = 1
UnusableEquipment["PRIEST"][WEAPON][THROWN] = 1
UnusableEquipment["PRIEST"][WEAPON][TWOHANDAXES] = 1
UnusableEquipment["PRIEST"][WEAPON][TWOHANDMACES] = 1
UnusableEquipment["PRIEST"][WEAPON][TWOHANDSWORDS] = 1
UnusableEquipment["PRIEST"][ARMOR][LEATHER] = 1
UnusableEquipment["PRIEST"][ARMOR][MAIL] = 1
UnusableEquipment["PRIEST"][ARMOR][PLATE] = 1
UnusableEquipment["PRIEST"][ARMOR][SHIELDS] = 1
UnusableEquipment["PRIEST"][ARMOR][LIBRAMS] = 1
UnusableEquipment["PRIEST"][ARMOR][IDOLS] = 1
UnusableEquipment["PRIEST"][ARMOR][TOTEMS] = 1
UnusableEquipment["PRIEST"][ARMOR][SIGILS] = 1

UnusableEquipment["ROGUE"][WEAPON][POLEARMS] = 1
UnusableEquipment["ROGUE"][WEAPON][STAVES] = 1
UnusableEquipment["ROGUE"][WEAPON][TWOHANDAXES] = 1
UnusableEquipment["ROGUE"][WEAPON][TWOHANDMACES] = 1
UnusableEquipment["ROGUE"][WEAPON][TWOHANDSWORDS] = 1
UnusableEquipment["ROGUE"][WEAPON][WANDS] = 1
UnusableEquipment["ROGUE"][ARMOR][MAIL] = 1
UnusableEquipment["ROGUE"][ARMOR][PLATE] = 1
UnusableEquipment["ROGUE"][ARMOR][SHIELDS] = 1
UnusableEquipment["ROGUE"][ARMOR][LIBRAMS] = 1
UnusableEquipment["ROGUE"][ARMOR][IDOLS] = 1
UnusableEquipment["ROGUE"][ARMOR][TOTEMS] = 1
UnusableEquipment["ROGUE"][ARMOR][SIGILS] = 1

UnusableEquipment["SHAMAN"][WEAPON][BOWS] = 1
UnusableEquipment["SHAMAN"][WEAPON][CROSSBOWS] = 1
UnusableEquipment["SHAMAN"][WEAPON][GUNS] = 1
UnusableEquipment["SHAMAN"][WEAPON][ONEHANDSWORDS] = 1
UnusableEquipment["SHAMAN"][WEAPON][POLEARMS] = 1
UnusableEquipment["SHAMAN"][WEAPON][THROWN] = 1
UnusableEquipment["SHAMAN"][WEAPON][TWOHANDSWORDS] = 1
UnusableEquipment["SHAMAN"][WEAPON][WANDS] = 1
UnusableEquipment["SHAMAN"][ARMOR][PLATE] = 1
UnusableEquipment["SHAMAN"][ARMOR][LIBRAMS] = 1
UnusableEquipment["SHAMAN"][ARMOR][IDOLS] = 1
UnusableEquipment["SHAMAN"][ARMOR][SIGILS] = 1

UnusableEquipment["WARLOCK"][WEAPON][BOWS] = 1
UnusableEquipment["WARLOCK"][WEAPON][CROSSBOWS] = 1
UnusableEquipment["WARLOCK"][WEAPON][FISTWEAPONS] = 1
UnusableEquipment["WARLOCK"][WEAPON][GUNS] = 1
UnusableEquipment["WARLOCK"][WEAPON][ONEHANDAXES] = 1
UnusableEquipment["WARLOCK"][WEAPON][ONEHANDMACES] = 1
UnusableEquipment["WARLOCK"][WEAPON][POLEARMS] = 1
UnusableEquipment["WARLOCK"][WEAPON][THROWN] = 1
UnusableEquipment["WARLOCK"][WEAPON][TWOHANDAXES] = 1
UnusableEquipment["WARLOCK"][WEAPON][TWOHANDMACES] = 1
UnusableEquipment["WARLOCK"][WEAPON][TWOHANDSWORDS] = 1
UnusableEquipment["WARLOCK"][ARMOR][LEATHER] = 1
UnusableEquipment["WARLOCK"][ARMOR][MAIL] = 1
UnusableEquipment["WARLOCK"][ARMOR][PLATE] = 1
UnusableEquipment["WARLOCK"][ARMOR][SHIELDS] = 1
UnusableEquipment["WARLOCK"][ARMOR][LIBRAMS] = 1
UnusableEquipment["WARLOCK"][ARMOR][IDOLS] = 1
UnusableEquipment["WARLOCK"][ARMOR][TOTEMS] = 1
UnusableEquipment["WARLOCK"][ARMOR][SIGILS] = 1

UnusableEquipment["WARRIOR"][WEAPON][WANDS] = 1
UnusableEquipment["WARRIOR"][ARMOR][LIBRAMS] = 1
UnusableEquipment["WARRIOR"][ARMOR][IDOLS] = 1
UnusableEquipment["WARRIOR"][ARMOR][TOTEMS] = 1
UnusableEquipment["WARRIOR"][ARMOR][SIGILS] = 1

--[[ Config UI --]]
function BagSaver:CreateConfigMenu()
	self.name = "BagSaver"
	self.okay = function (self) return end
	self.cancel = function (self) return end

	local title, notes = select(2, GetAddOnInfo('BagSaver'))
	local text = self:CreateFontString(nil, 'ARTWORK', 'GameFontNormalLarge')
	text:SetPoint('TOPLEFT', 16, -16)
	text:SetText(title)

	local desc = self:CreateFontString(nil, 'ARTWORK', 'GameFontHighlightSmall')
	desc:SetHeight(32)
	desc:SetPoint('TOPLEFT', text, 'BOTTOMLEFT', 0, -8)
	desc:SetPoint('RIGHT', self, -32, 0)
	desc:SetNonSpaceWrap(true)
	desc:SetJustifyH('LEFT')
	desc:SetJustifyV('TOP')
	desc:SetText(notes)

	local autoSell = self:CreateCheckButton("Auto-Sell", self,"Enable automatic selling of \"junk\" to vendors", "autoSell", 'InterfaceOptionsCheckButtonTemplate')
	autoSell:SetPoint('TOPLEFT', 10, -75)

	local autoSellPrompt = self:CreateCheckButton("Prompt before Auto-Sell", self,"Prompt before automatically selling anything to a vendor", "autoSellPrompt", 'InterfaceOptionsSmallCheckButtonTemplate')
	autoSellPrompt:SetPoint('TOPLEFT', 150, -75)
	BagSaver:SetupDependentControl(autoSell,autoSellPrompt)

	local autoSellGrayItems = self:CreateCheckButton("Gray Items", self,"Consider gray items for Auto-Sale", "autoSellGrayItems", 'InterfaceOptionsCheckButtonTemplate')
	autoSellGrayItems:SetPoint('TOPLEFT', 20, -100)
	BagSaver:SetupDependentControl(autoSell,autoSellGrayItems)

	local autoSellUnusableBoundItems = self:CreateCheckButton("Unusable Soulbound Items", self,"Consider Soulbound items that you could *never* equip for Auto-Sale", "autoSellUnusableBoundItems", 'InterfaceOptionsCheckButtonTemplate')
	autoSellUnusableBoundItems:SetPoint('TOPLEFT', 20, -125)
	BagSaver:SetupDependentControl(autoSell,autoSellUnusableBoundItems)

	InterfaceOptions_AddCategory(self)
end

function BagSaver:CreateCheckButton(name, parent, tooltipText, configKey, template)
	--print("Setting name to " .. parent:GetName() .. name)
	local button = CreateFrame('CheckButton', parent:GetName() .. name, parent, template)
	getglobal(button:GetName() .. 'Text'):SetText(name)
	--print("Frame name is " .. button:GetName())
	--button:SetScript('OnShow', function(self) self:SetChecked(BagSaver.GetConfigValue(configKey)) end)
	button:SetChecked(BagSaver.GetConfigValue(configKey))
	button:SetScript('OnClick', function(self)
		if self:GetChecked() then
			BagSaver.SetConfigValue(configKey, true)
		else 
			BagSaver.SetConfigValue(configKey, false)
		end
		if ( self.dependentControls ) then
			if ( self:GetChecked() ) then
				for _, control in pairs(self.dependentControls) do
					control:Enable()
				end
			else
				for _, control in pairs(self.dependentControls) do
					control:Disable()
				end
			end
		end
	end)
	button:SetScript('OnEnter', function(self)
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		GameTooltip:SetText(tooltipText, nil, nil, nil, 1, 1)
		GameTooltip:Show() end)
	button:SetScript('OnLeave', function(self) GameTooltip:Hide() end)

	return button
end

function BagSaver:SetupDependentControl (dependency, control)
	if ( not dependency ) then
		return
	end

	assert(control)

	dependency.dependentControls = dependency.dependentControls or {}
	tinsert(dependency.dependentControls, control)

	if ( control.type ~= CONTROLTYPE_DROPDOWN ) then
		control.Disable = function (self) getmetatable(self).__index.Disable(self) _G[self:GetName().."Text"]:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b) end
		control.Enable = function (self) getmetatable(self).__index.Enable(self) _G[self:GetName().."Text"]:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b) end
		if dependency:GetChecked() then
			control:Enable()
		else
			control:Disable()
		end
	else
		control.Disable = function (self) UIDropDownMenu_DisableDropDown(self) end
		control.Enable = function (self) UIDropDownMenu_EnableDropDown(self) end
	end

end